
// Include Libraries
#include "Arduino.h"
#include "DHT.h"
#include "LiquidCrystal_PCF8574.h"
#include "SoilMoisture.h"
#include "SolenoidValve.h"


// Pin Definitions
#define DHT_PIN_DATA	2
#define SOILMOISTURE5V_PIN_SIG	A3
#define SOLENOIDVALVE_PIN_COIL1	3
#define isWatering
#define tooDry
#define waterDurationSeconds
#define lastWateredTime


// There are several different versions of the LCD I2C adapter, each might have a different address.
//#define LCD_ADDRESS 0x3F
#define LCD_ADDRESS 0x27

// Define LCD characteristics
#define LCD_ROWS 2
#define LCD_COLUMNS 16
#define SCROLL_DELAY 150
#define BACKLIGHT 255
// object initialization
DHT dht(DHT_PIN_DATA);
LiquidCrystal_PCF8574 lcdI2C;
SoilMoisture soilMoisture5v(SOILMOISTURE5V_PIN_SIG);
SolenoidValve solenoidValve(SOLENOIDVALVE_PIN_COIL1);



// define vars for testing menu
const int timeout = 10000;       //define timeout of 10 sec
char menuOption = 0;
long time0;

// Setup the essentials for your circuit to work. It runs first every time your circuit is powered with electricity.
void setup() 
{
    // Setup Serial which is useful for debugging
    // Use the Serial Monitor to view printed messages
    Serial.begin(9600);
    while (!Serial) ; // wait for serial port to connect. Needed for native USB
    Serial.println("start");
    
    dht.begin();
    // initialize the lcd
    lcdI2C.begin(LCD_COLUMNS, LCD_ROWS, LCD_ADDRESS, BACKLIGHT); 
    menuOption = menu();
    
}

// Main logic of your circuit. It defines the interaction between the components you selected. After setup, it runs over and over again, in an eternal loop.
void loop() 
{
    
    
    if(menuOption == '1') {
    // DHT22/11 Humidity and Temperature Sensor - Test Code
    // Reading humidity in %
    float dhtHumidity = dht.readHumidity();
    // Read temperature in Fahrenheit, for Celsius use .readTempC()
    float dhtTempC = dht.readTempF();
    Serial.print(F("Humidity: ")); Serial.print(dhtHumidity); Serial.print(F(" [%]\t"));
    Serial.print(F("Temp: ")); Serial.print(dhtTempC); Serial.println(F(" [F]"));

    }
    else if(menuOption == '2') {
    // LCD Display Screen 16x2 I2C - Test Code
    // The LCD Screen will display the text of your choice.
    lcdI2C.clear();                          // Clear LCD screen.
    lcdI2C.print("  Testing  ");                   // Print print String to LCD on first line
    lcdI2C.selectLine(2);                    // Set cursor at the begining of line 2
    lcdI2C.print("     LCD  ");                   // Print print String to LCD on second line
    delay(1000);

    }
    else if(menuOption == '3') {
    // Soil Moisture Sensor - Test Code
    int soilMoisture5vVal = soilMoisture5v.read();
    Serial.print(F("Val: ")); Serial.println(soilMoisture5vVal);

    }
    else if(menuOption == '4') {
    // 12V Solenoid Valve - 3/4'' - Test Code
    // The solenoid valve will turn on and off for 500ms (0.5 sec)
    solenoidValve.on(); // 1. turns on
    delay(500);       // 2. waits 500 milliseconds (0.5 sec). Change the value in the brackets (500) for a longer or shorter delay in milliseconds.
    solenoidValve.off();// 3. turns off
    delay(500);       // 4. waits 500 milliseconds (0.5 sec). Change the value in the brackets (500) for a longer or shorter delay in milliseconds.

    }
    
    
    
    if (millis() - time0 > timeout)
    {
        menuOption = menu();
    }

{
  /* *********************************************** */
  /* Read all the values from the components we have */
  /* *********************************************** */
  // DHT22/11 Humidity and Temperature Sensor - Read humidity in %
  float humidity = dht.readHumidity();
  // Read temperature in Celsius, for Fahrenheit use .readTempF()
  float tempC = dht.readTempC();

  // Soil Moisture Sensor - Test Code
  int soilMoisture5vVal = soilMoisture5v.read();

  /* *********************************************** */
  /* Water the plant if necessary                    */
  /* *********************************************** */
  isWatering = false;
  if (soilMoisture5vVal < tooDry) {
    solenoidValve.on();
    delay(waterDurationSeconds * 1000);
    solenoidValve.off();

    isWatering = true;
    // millis is the time since starting up. See https://www.arduino.cc/reference/en/language/functions/time/millis/
    lastWateredTime = millis();
  }

  /* *********************************************** */
  /* Show the values on the LCD                      */
  /* *********************************************** */
  // Note: sprintf is a function to format a string
  // We have floats for the humidity and temp. Arduino's sprintf function doesn't include support for floats.
  // We'll simplify things by converting to integers. We're aiming for a string like "12% 16C nV" to show humidity, temp, and soil "Voltage".
  // We'll format an array of characters (a string) with the values using sprintf, then print that with the LCD library
  // Line 1 can be sensor values, and hopefully we can get the rest of the feedback into line 2

  char buffer[16];
  lcdI2C.clear();
  sprintf(buffer, "%02d\% %02dC %dV", int(humidity), int(tempC), soilMoisture5vVal);
  lcdI2C.print(buffer);
  lcdI2C.selectLine(2);
  sprintf(buffer, "%s %d Mins", (isWatering ? "Watering" : "        "), (millis() - lastWateredTime)/1000/60);
  lcdI2C.print(buffer);

  // Wait for a second before updating again
  delay(1000);
}
}



// Menu function for selecting the components to be tested
// Follow serial monitor for instrcutions
char menu()
{

    Serial.println(F("\nWhich component would you like to test?"));
    Serial.println(F("(1) DHT22/11 Humidity and Temperature Sensor"));
    Serial.println(F("(2) LCD Display Screen 16x2 I2C"));
    Serial.println(F("(3) Soil Moisture Sensor"));
    Serial.println(F("(4) 12V Solenoid Valve - 3/4''"));
    Serial.println(F("(menu) send anything else or press on board reset button\n"));
    while (!Serial.available());

    // Read data from serial monitor if received
    while (Serial.available()) 
    {
        char c = Serial.read();
        if (isAlphaNumeric(c)) 
        {
            if(c == '1') 
    			Serial.println(F("Now Testing DHT22/11 Humidity and Temperature Sensor"));
    		else if(c == '2') 
    			Serial.println(F("Now Testing LCD Display Screen 16x2 I2C"));
    		else if(c == '3') 
    			Serial.println(F("Now Testing Soil Moisture Sensor"));
    		else if(c == '4') 
    			Serial.println(F("Now Testing 12V Solenoid Valve - 3/4''"));
            else
            {
                Serial.println(F("illegal input!"));
                return 0;
            }
            time0 = millis();
            return c;
        }
    }
}
